'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _loaderUtils = require('loader-utils');

var _schemaUtils = require('schema-utils');

var _schemaUtils2 = _interopRequireDefault(_schemaUtils);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var classNameRegex = /className=\"([0-9a-zA-Z\-\_\s]*)\"/ig;
var classNamesRegex = /classnames\(((.|\s)*?)\)/img;
var stringBetweenQuotesRegex = /(["'])(\\?.)*?\1/img;

var ignore = [];
var fs = require('fs');
var async = require('async');

var modifiedSource = null;

var schema = {
    type: 'object',
    properties: {
        fileName: {
            type: 'array'
        },
        prefix: {
            type: 'string'
        }
    }
};

function loader(source, inputSourceMap) {

    var options = (0, _loaderUtils.getOptions)(this);
    var self = this;

    var callback = this.async();

    var cssContent = '';

    (0, _schemaUtils2.default)(schema, options, 'React ClassName Prefix Loader');

    async.eachSeries(
    // Pass items to iterate over
    options.fileName,
    // Pass iterator function that is called for each item
    function (filename, cb) {
        fs.readFile(filename, "utf8", function (err, content) {
            var filePath = _path2.default.resolve(filename);

            self.addDependency(filePath);

            cssContent = cssContent.concat(content);
            cssContent = cssContent.concat('\n');
            // Calling cb makes it go to the next item.
            cb(err);
        });
    },
    // Final callback after each item has been iterated over.
    function (err) {
        checkFiles(cssContent, options.prefix, source, function () {
            callback(null, modifiedSource, inputSourceMap);
        });
    });
}

function checkFiles(content, prefix, source, callback) {
    if (prefix) {
        // Process classes with classnames module
        var classNamesMatches = source.match(classNamesRegex);

        if (classNamesMatches) {
            classNamesMatches.map(function (item) {
                item = item.replace(/\s+/g, '');

                var classNamesMatchesStrings = item.match(stringBetweenQuotesRegex);

                if (classNamesMatchesStrings) {
                    classNamesMatchesStrings.map(function (item) {
                        modifiedSource = source.replace(new RegExp(item, 'g'), function (text) {
                            var replaceResult = "'" + prefix + text.replace(/['"]/g, '') + "'";
                            ignore.push(replaceResult);
                            return replaceResult;
                        });
                    });
                }
            });
        }

        modifiedSource = source.replace(classNameRegex, function (text, classNames) {
            var attr = text.match(/classname/ig);
            if (attr && attr[0]) {
                attr = attr[0];
            } else {
                attr = 'className';
            }

            var prefixedClassNames = classNames.split(' ').map(function (className) {
                if (className.indexOf(prefix) >= 0 || ignoreClassName(className) || content.indexOf(className) < 0) {
                    return className;
                }

                return '' + prefix + className;
            }).join(' ');

            return attr + '=\'' + prefixedClassNames + '\'';
        });
    }

    callback();
}

function ignoreClassName(className) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    return classMatchesTest(className, options.ignore) || className.trim().length === 0 || /^[A-Z-]/.test(className) || ignore.findIndex(function (item) {
        return className === item;
    }) >= 0;
}

function classMatchesTest(className, ignore) {
    if (!ignore) return false;

    className = className.trim();

    if (ignore instanceof RegExp) return ignore.exec(className);

    if (Array.isArray(ignore)) {
        return ignore.some(function (test) {
            if (test instanceof RegExp) return test.exec(className);

            return className === test;
        });
    }

    return className === ignore;
}

exports.default = loader;
module.exports = exports['default'];